package com.tanpu.community.manager;

import com.tanpu.common.api.CommonResp;
import com.tanpu.common.exception.BizException;
import com.tanpu.common.util.JsonUtil;
import com.tanpu.community.api.beans.qo.AttachmentDetailVo;
import com.tanpu.community.api.beans.qo.ThemeContentQo;
import com.tanpu.community.api.beans.qo.ThemeQo;
import com.tanpu.community.api.beans.req.theme.CreateThemeReq;
import com.tanpu.community.api.beans.req.theme.ForwardThemeReq;
import com.tanpu.community.api.beans.req.theme.ThemeListByTopicReq;
import com.tanpu.community.api.beans.req.theme.ThemeListReq;
import com.tanpu.community.api.beans.vo.feign.CoursePackageDetail;
import com.tanpu.community.api.beans.vo.feign.ProductInfoVO;
import com.tanpu.community.api.beans.vo.feign.ShortVideoBaseInfoResp;
import com.tanpu.community.api.beans.vo.feign.ZhiboDetailVO;
import com.tanpu.community.api.enums.*;
import com.tanpu.community.dao.entity.community.BlackListEntity;
import com.tanpu.community.dao.entity.community.HomePageEntity;
import com.tanpu.community.dao.entity.community.ThemeAttachmentEntity;
import com.tanpu.community.dao.entity.community.ThemeEntity;
import com.tanpu.community.feign.course.FeignClientForCourse;
import com.tanpu.community.feign.product.FeignClientForProducts;
import com.tanpu.community.feign.tanpuroom.FeignClientForTanpuroom;
import com.tanpu.community.feign.zhibo.FeignClientForZhibo;
import com.tanpu.community.service.*;
import com.tanpu.community.service.other.*;
import com.tanpu.community.util.ConvertUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Service
public class ThemeManager {
    @Resource
    private ThemeService themeService;

    @Autowired
    private CollectionService collectionService;

    @Autowired
    private CommentService commentService;

    @Autowired
    private FollowRelService followRelService;

    @Autowired
    private BlackListService blackListService;

    @Autowired
    private FinProResService finProResService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private ZhiboService zhiboService;

    @Autowired
    private CurriculumResService curriculumResService;

    @Autowired
    private OrderFlowService orderFlowService;

    @Autowired
    private ThemeAttachmentService themeAttachmentService;

    @Autowired
    private HomePageService homePageService;

    @Resource
    private FeignClientForTanpuroom feignClientForTanpuroom;

    @Resource
    private FeignClientForZhibo feignClientForZhibo;

    @Resource
    private FeignClientForProducts feignClientForProducts;

    @Autowired
    private FeignClientForCourse feignClientForCourse;


    public void publishTheme(CreateThemeReq req, String userId) {

        //TODO 敏感词过滤

        ThemeEntity themeEntity = new ThemeEntity();
        BeanUtils.copyProperties(req, themeEntity);
        themeEntity.setAuthorId(userId);
        themeEntity.setContent(JsonUtil.toJson(req.getContent()));

        themeService.insertTheme(themeEntity);


        List<ThemeAttachmentEntity> themeAttachments = ConvertUtil.themeReqToAttachmentList(req, themeEntity.getThemeId());
        themeAttachmentService.insertList(themeAttachments);





    }

    // 返回推荐主题文章
    public List<ThemeQo> queryThemeList(ThemeListReq req, String userId) {
        List<ThemeEntity> themeEntities;
        if (ThemeListTypeEnum.RECOMMEND.getCode().equals(req.getType())) {
            // TODO:推荐
            themeEntities = themeService.selectAll(req.getLastId(), req.getPageSize());
        } else {
            List<String> fansList = followRelService.queryFansByFollowerId(userId);
            themeEntities = themeService.queryByFans(fansList, req.getLastId(), req.getPageSize());
        }
        List<ThemeQo> themeQos = convertEntityToQo(themeEntities, userId);
        return themeQos;
    }


    public List<ThemeQo> queryByTopic(ThemeListByTopicReq req, String userId) {
        List<ThemeEntity> themeEntities = themeService.queryByTopic(req.getTopicId(), req.getLastId(), req.getPageSize());
        List<ThemeQo> themeQos = convertEntityToQo(themeEntities, userId);
        return themeQos;
    }

    //查询正文
    public ThemeQo getDetail(String themeId, String userId) {

        ThemeEntity themeEntity = themeService.queryByThemeId(themeId);
        if (themeEntity == null) {
            throw new BizException("找不到帖子id：" + themeId);
        }
        ThemeQo themeQo = ConvertUtil.themeEntityToQo(themeEntity);
        buildMainTestExtraInfo(themeQo, userId);
        return themeQo;
    }

    // 点赞
    public void like(String themeId, String userId) {
        collectionService.addIfNotExist(themeId, userId, CollectionTypeEnum.LIKE_THEME);
    }

    public void unlike(String themeId, String userId) {
    }


    //转发
    public void forward(ForwardThemeReq req, String userId) {
        ThemeEntity targetTheme = themeService.queryByThemeId(req.getFormerThemeId());
        ThemeEntity newTheme = ThemeEntity.builder()
                .content(req.getContent())
                .topicId(req.getTopicId())
                .formerThemeId(req.getFormerThemeId())
                .authorId(userId)
                .themeType(ThemeTypeEnum.DISCUSSION.getCode())
                .build();

        themeService.insertTheme(newTheme);
    }


    //收藏
    public void favorite(String themeId, String userId) {
        collectionService.addIfNotExist(themeId, userId, CollectionTypeEnum.BOOK_THEME);
    }

    public void unFavorite(String themeId, String userId) {
        collectionService.delete(themeId, userId, CollectionTypeEnum.BOOK_THEME);
    }

    //投诉（主题）
    public void complaint(String themeId, String user) {
        //TODO
    }

    //    屏蔽(用户)
    public void blockUser(String blockUser, String userId) {
        BlackListEntity selectOne = blackListService.selectOne(blockUser, userId, BlockTypeEnum.USER.getCode());
        if (selectOne == null) {
            blackListService.addBlock(blockUser, userId, BlockTypeEnum.USER);
        }
    }

    // 解除屏蔽(用户)
    public void unblockUser(String blockUser, String userId) {
        //todo
    }

    // 屏蔽(主题)
    public void blockTheme(String themeId, String userId) {
        blackListService.addBlock(themeId, userId, BlockTypeEnum.THEME);
        BlackListEntity selectOne = blackListService.selectOne(themeService.queryByThemeId(themeId).getAuthorId(), userId, BlockTypeEnum.USER.getCode());
        if (selectOne == null) {
            blackListService.addBlock(themeService.queryByThemeId(themeId).getAuthorId(), userId, BlockTypeEnum.USER);
        }
    }

    // 解除屏蔽(主题)
    public void unblockTheme(String themeId, String userId) {
        blackListService.removeBlackList(themeId, userId, BlockTypeEnum.USER);
    }


    //主题Entity转QO
    private List<ThemeQo> convertEntityToQo(List<ThemeEntity> themeEntities, String userId) {
        List<ThemeQo> themeQos = ConvertUtil.themeEntitiesToDTOs(themeEntities);
        for (ThemeQo themeQO : themeQos) {
            buildThemeQoExtraInfo(userId, themeQO);
        }
        return themeQos;
    }


    //组装主题列表
    private void buildThemeQoExtraInfo(String userId, ThemeQo themeQO) {
        //附件列表
        String themeId = themeQO.getThemeId();
        List<ThemeContentQo> content = themeQO.getContent();
        for (ThemeContentQo themeContentQo : content) {
            transferAttachment(themeContentQo,userId);
        }

        //迄今时间
        themeQO.setUpToNowTime(calUpToNowTime(themeQO.getCreateTime()));
        //是否关注作者
        String authorId = themeQO.getAuthorId();
        Set<String> fansSet = new HashSet<>(followRelService.queryFansByFollowerId(userId));
        themeQO.setFollow(fansSet.contains(authorId));
        //用户信息：头像、昵称
        HomePageEntity userEntity = homePageService.selectByUserId(userId);
        themeQO.setUserImg(userEntity.getHeadImg());
        themeQO.setNickName(userEntity.getNickName());
        //热点数据:点赞，收藏，转发
        Integer likeCount = collectionService.getCountByTypeAndId(themeId, CollectionTypeEnum.LIKE_THEME);
        Integer bookCount = collectionService.getCountByTypeAndId(themeId, CollectionTypeEnum.BOOK_THEME);
        Integer forwardCount = themeService.getForwardCountById(themeId);
        themeQO.setCommentCount(bookCount);
        themeQO.setLikeCount(likeCount);
        themeQO.setForwardCount(forwardCount);
    }

    //组装正文详情
    private void buildMainTestExtraInfo(ThemeQo themeQo, String userId) {
        //附件列表
        String themeId = themeQo.getThemeId();
        List<ThemeContentQo> content = themeQo.getContent();
        for (ThemeContentQo themeContentQo : content) {
            transferAttachment(themeContentQo,userId);
        }
        //迄今时间
        themeQo.setUpToNowTime(calUpToNowTime(themeQo.getCreateTime()));
        //用户信息：头像、昵称
        HomePageEntity author = homePageService.selectByUserId(userId);
        themeQo.setUserImg(author.getHeadImg());
        themeQo.setNickName(author.getNickName());

        return;
    }


    public void transferAttachment(ThemeContentQo themeContentQo, String userId) {
        String type = themeContentQo.getType();
        if (RelTypeEnum.FUND.type.equals(type)){
            //产品
            String productId = themeContentQo.getValue();
            CommonResp<ProductInfoVO> productDetailInfo = feignClientForProducts.getProductDetailInfo(productId);
            themeContentQo.setDetail(AttachmentDetailVo.builder().productInfoVO(productDetailInfo.getData()).build());

        }else if (RelTypeEnum.LIVE.type.equals(type)){
            //直播
            String zhiboId = themeContentQo.getValue();
            CommonResp<ZhiboDetailVO> zhiboDetailVOCommonResp = feignClientForZhibo.queryZhiboDetail(zhiboId);
            themeContentQo.setDetail(AttachmentDetailVo.builder().zhiboDetailVO(zhiboDetailVOCommonResp.getData()).build());

        }else if (RelTypeEnum.SHORT_VIDEO.type.equals(type)){
            //短视频
            String videoId = themeContentQo.getValue();
            String[] attachmentIds = new String[]{videoId};
            CommonResp<List<ShortVideoBaseInfoResp>> shortVideoBaseInfo =
                    feignClientForTanpuroom.getShortVideoBaseInfo(Arrays.asList(attachmentIds));
            themeContentQo.setDetail(AttachmentDetailVo.builder().shortVideoBaseInfoResp(shortVideoBaseInfo.getData().get(0)).build());

        }else if (RelTypeEnum.PACKAGE.type.equals(type)){
            //课程
            String packageId = themeContentQo.getValue();
            CommonResp<CoursePackageDetail> courseIdDetailInfo = feignClientForCourse.getCoursePackageDetailInfo(packageId);
            themeContentQo.setDetail(AttachmentDetailVo.builder().coursePackageDetail(courseIdDetailInfo.getData()).build());
        }


    }


    //计算迄今时间
    private String calUpToNowTime(LocalDateTime start) {
        Duration between = Duration.between(start, LocalDateTime.now());
        long duration = between.toMinutes();
        if (duration < 1) {
            return "刚刚";
        } else if (duration < 60) {
            return duration + "分钟前";
        } else if (duration < 60 * 24) {
            return duration / 60 + "小时前";
        } else if (start.getYear() == LocalDateTime.now().getYear()) {
            return start.format(DateTimeFormatter.ofPattern("MM-dd HH:mm:ss"));
        }
        return start.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }


}
